Class: Concord::Metadata
- Inherits:
-
Object
- Object
- Concord::Metadata
- Defined in:
- lib/concord.rb
Overview
Wrapper over the ComputationMetadata object from the thrift interface
Instance Attribute Summary collapse
-
#istreams ⇒ Object
Returns the value of attribute istreams.
-
#name ⇒ Object
Returns the value of attribute name.
-
#ostreams ⇒ Object
Returns the value of attribute ostreams.
Instance Method Summary collapse
-
#initialize(name: nil, istreams: [], ostreams: []) ⇒ Metadata
constructor
Initialize a new
Metadataobject as either aStringidentifying the stream or anArrayof length two, in which the first parameter is theStringidentifier and the second parameter is aStreamGroupingdefining how incoming records should be routed.
Constructor Details
#initialize(name: nil, istreams: [], ostreams: []) ⇒ Metadata
Initialize a new Metadata object as either a String identifying the stream or an Array of length two, in which the first parameter is the String identifier and the second parameter is a StreamGrouping defining how incoming records should be routed. computation may produce records on.
22 23 24 25 26 |
# File 'lib/concord.rb', line 22 def initialize(name: nil, istreams: [], ostreams: []) self.name = name self.istreams = istreams self.ostreams = ostreams end |
Instance Attribute Details
#istreams ⇒ Object
Returns the value of attribute istreams.
11 12 13 |
# File 'lib/concord.rb', line 11 def istreams @istreams end |
#name ⇒ Object
Returns the value of attribute name.
11 12 13 |
# File 'lib/concord.rb', line 11 def name @name end |
#ostreams ⇒ Object
Returns the value of attribute ostreams.
11 12 13 |
# File 'lib/concord.rb', line 11 def ostreams @ostreams end |